/******************************************************************************************
 *
 * CHECK_DNS.C
 *
 * Program: DNS plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: 08-14-1999
 *
 * Command line: CHECK_DNS <query_address> [dns_server]
 *
 * Description:
 *
 * This program will use the nslookup program to obtain the IP address for a given
 * host name.  A optional DNS server may be specified.  If no DNS server is specified, the
 * default server(s) for the system are used.
 *
 * Return Values:
 * 	OK		The DNS query was successful.  IP address of the host is returned.
 *	WARNING		The DNS server responded, but could not fulfill the lookup request.
 *	CRITICAL	The DNS server is not responding or encountered an error.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************************/

#include "../common/config.h"
#include "../common/locations.h"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <time.h>
#include <sys/types.h>
#include <sys/wait.h>

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define MAX_INPUT_BUFFER	512

#define TEMP_FILE_NAME		"check_dns.tmp"
#define TEMP_FILE_NAME1		"check_dns.tmp.1"
#define TEMP_FILE_NAME2		"check_dns.tmp.2"
char temp_file[MAX_INPUT_BUFFER]="";
char temp_file1[MAX_INPUT_BUFFER]="";
char temp_file2[MAX_INPUT_BUFFER]="";

time_t start_time,end_time;

int my_system(char *);



int main(int argc, char **argv){
	char query_address[256];
	char dns_server[256];
	char command_line[256];
	FILE *fp;
	char input_buffer[MAX_INPUT_BUFFER];
	char output[256];
	char *temp_buffer;
	int result;

	if(argc<2||argc>3){
		printf("Incorrect number of arguments supplied\n");
		printf("\n");
		printf("DNS plugin for NetSaint\n");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 08-14-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <query_address> [dns_server]\n",argv[0]);
		printf("\n");
		printf("Note:\n");
		printf("<query_address> = The name of the address you want to query\n");
		printf("[dns_server]    = Optional DNS server you want to use for the lookup\n");
		printf("\n");
		printf("This plugin will use the nslookup program to obtain the IP address\n");
		printf("for the given host/domain query.  A optional DNS server to use may\n");
		printf("be specified.  If no DNS server is specified, the default server(s)\n");
		printf("specified in /etc/resolv.conf will be used.\n");
		printf("\n");
		return STATE_UNKNOWN;
	        }

	/* get the query address */
	strcpy(query_address,argv[1]);

	/* get the dns server (optional) */
	if(argc==3)
		strcpy(dns_server,argv[2]);
	else
		strcpy(dns_server,"");

	/* get the path/file names of the temp files to use */
	snprintf(temp_file,sizeof(temp_file),"%s/%s",DEFAULT_TEMP_FILE_PATH,TEMP_FILE_NAME);
	temp_file[sizeof(temp_file)-1]='\x0';
	snprintf(temp_file1,sizeof(temp_file1),"%s/%s",DEFAULT_TEMP_FILE_PATH,TEMP_FILE_NAME1);
	temp_file1[sizeof(temp_file1)-1]='\x0';
	snprintf(temp_file2,sizeof(temp_file2),"%s/%s",DEFAULT_TEMP_FILE_PATH,TEMP_FILE_NAME2);
	temp_file2[sizeof(temp_file2)-1]='\x0';

	/* run the command */
	sprintf(command_line,"%s %s %s > %s 2> %s ; cat %s %s > %s",NSLOOKUP_COMMAND,query_address,dns_server,temp_file1,temp_file2,temp_file1,temp_file2,temp_file);
	time(&start_time);
	my_system(command_line);
	time(&end_time);

	result=STATE_UNKNOWN;
	strcpy(output,"");

	/* open the temp file */
	fp=fopen(temp_file,"r");
	if(fp==NULL){
		remove(temp_file);
		remove(temp_file1);
		remove(temp_file2);
		printf("Error: Could not open temp file for reading\n");
		return STATE_UNKNOWN;
	        }

	while(fgets(input_buffer,MAX_INPUT_BUFFER-1,fp)){

		/* the server is responding, we just got the host name... */
		if(strstr(input_buffer,"Name:")){

			/* get the host address */
			if(!fgets(input_buffer,MAX_INPUT_BUFFER-1,fp))
				break;

			temp_buffer=strtok(input_buffer,":");
			temp_buffer=strtok(NULL,":");
			if(temp_buffer!=NULL)
				strcpy(output,temp_buffer+2);
			else
				strcpy(output,"Unknown error (plugin)");

			result=STATE_OK;
			break;
			}


		/* the DNS lookup timed out */
		else if(strstr(input_buffer,"Timed out")){
			strcpy(output,"Timed out");
			result=STATE_WARNING;
			break;
			}

		/* DNS server is not running... */
		else if(strstr(input_buffer,"No response from server")){
			strcpy(output,"No response from server");
			result=STATE_CRITICAL;
			break;
			}

		/* Host name is valid, but server doesn't have records... */
		else if(strstr(input_buffer,"No records")){
			strcpy(output,"No records");
			result=STATE_WARNING;
			break;
			}

		/* Host or domain name does not exist */
		else if(strstr(input_buffer,"Non-existent")){
			strcpy(output,"Non-existent host/domain");
			result=STATE_CRITICAL;
			break;
			}

		/* Connection was refused */
		else if(strstr(input_buffer,"Connection refused")){
			strcpy(output,"Connection refused");
			result=STATE_CRITICAL;
			break;
			}

		/* Network is unreachable */
		else if(strstr(input_buffer,"Network is unreachable")){
			strcpy(output,"Network is unreachable");
			result=STATE_CRITICAL;
			break;
			}

		/* Internal server failure */
		else if(strstr(input_buffer,"Server failure")){
			strcpy(output,"Server failure");
			result=STATE_CRITICAL;
			break;
			}

		/* DNS server refused to service request */
		else if(strstr(input_buffer,"Refused")){
			strcpy(output,"Refused");
			result=STATE_CRITICAL;
			break;
			}

		/* Request error */
		else if(strstr(input_buffer,"Format error")){
			strcpy(output,"Format error");
			result=STATE_WARNING;
			break;
			}


		}

	/* close the temp file */
	fclose(fp);
	
	if(result==STATE_OK)
		printf("DNS ok - %d seconds response time\n\nAddress(es) is/are %s",(int)(end_time-start_time),output);
	else if(result==STATE_UNKNOWN){
		printf("DNS error\n");
		result=STATE_CRITICAL;
	        }
	else
		printf("%s\n",output);

	/* remove the temp files */
	remove(temp_file);
	remove(temp_file1);
	remove(temp_file2);

	return result;
        }



int my_system(char *command){
        int pid;
	int status;
	int result;
	extern int errno;
	extern char **environ;


	/* if no command was passed, return with no error */
	if(command==NULL)
	        return STATE_OK;

	/* fork */
	pid=fork();

	/* return an error if we couldn't fork */
	if(pid==-1)
	        return STATE_UNKNOWN;  

	/* execute the command in the child process */
        if (pid==0){
		char *argv[4];
		argv[0]="sh";
		argv[1]="-c";
		argv[2]=command;
		argv[3]=0;
		execve("/bin/sh",argv,environ);
		exit(STATE_UNKNOWN);
	        }

	/* wait for the command to finish */
        do{
                if (waitpid(pid, &status, 0) == -1) {
		        if (errno != EINTR)
                                return STATE_UNKNOWN;
                        } 

		/* get the exit code returned from the program */
		result=WEXITSTATUS(status);

		/* check the bounds for the return code */
		if(result<0||result>2)
			return STATE_UNKNOWN;

		return result;
           } while(1);

        return STATE_OK;
        }

