/**********************************************************************************************
 *
 * CHECK_TIME.C
 *
 * Program: Network time server plugin for NetSaint
 * License: GPL
 * Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)
 *
 * Last Modified: 07-21-1999
 *
 * Command line: CHECK_TIME <host_address> [-p port] [-wd warn_diff] [-cd crit_diff] \ 
 *			    [-to to_sec] [-wt warn_time] [-ct crit_time]
 *
 * Description:
 *
 * This plugin will attempt to connect to the specified port
 * on the host.  Successul connects return STATE_OK, refusals
 * and timeouts return STATE_CRITICAL, other errors return
 * STATE_UNKNOWN.
 *
 * License Information:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 **********************************************************************************************/

#include "../common/config.h"
#include <stdio.h>		/* obligatory includes */
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <time.h>
#include <signal.h>

#define OK		0
#define ERROR		-1

#define TRUE		1
#define FALSE		0

#define	STATE_CRITICAL 2	/* service state return codes */
#define STATE_WARNING  1
#define STATE_OK       0
#define STATE_UNKNOWN  -1

#define TIME_PORT	37
#define TIME_PROTO	"tcp"

#define MAX_INPUT_BUFFER	256

#define DEFAULT_TIMEOUT	10	/* timeout after 10 seconds */

#define UNIX_EPOCH	2208988800UL

time_t start_time,end_time;
time_t local_time;
unsigned long server_time, raw_server_time;
time_t diff_time;
int warning_time=0;
int check_warning_time=FALSE;
int critical_time=0;
int check_critical_time=FALSE;
unsigned long warning_diff=0;
int check_warning_diff=FALSE;
unsigned long critical_diff=0;
int check_critical_diff=FALSE;

int my_connect(char *address,int port,char *protocol,int *sd);
int process_arguments(int,char **);
void alarm_handler(int);

int server_port=TIME_PORT;
int timeout=DEFAULT_TIMEOUT;
char server_name[256];


int main(int argc, char **argv){
	int sd;
	int result;
	char buffer[MAX_INPUT_BUFFER];

	result=process_arguments(argc,argv);

	if(result!=OK){

		printf("Invalid command arguments supplied\n");
		printf("\n");
		printf("Network time server plugin for NetSaint\n");
		printf("Copyright (c) 1999 Ethan Galstad (netsaint@linuxbox.com)\n");
		printf("Last Modified: 07-21-1999\n");
		printf("License: GPL\n");
		printf("\n");
		printf("Usage: %s <host_address> [-p port] [-wd warn_diff] [-cd crit_diff] [-wt warn_time] [-ct crit_time] [-to to_sec]\n",argv[0]);
		printf("\n");
		printf("Options:\n");
		printf(" [port]      = Port number to connect to - default is %d\n",TIME_PORT);
		printf(" [warn_diff] = Time difference in seconds necessary to result in a warning status\n");
		printf(" [crit_diff] = Time difference in seconds necessary to result in a critical status\n");
		printf(" [warn_time] = Response time in seconds necessary to result in a warning status\n");
		printf(" [crit_time] = Response time in seconds necessary to result in a critical status\n");
		printf(" [to_sec]    = Number of seconds before connection attempt times out - default is %d seconds\n",DEFAULT_TIMEOUT);
		printf("\n");
		printf("This plugin will attempt to connect to a TIME port on the specified host.\n");
		printf("If possible, supply an IP address for the host address, as this will bypass the DNS lookup.\n");
		printf("\n");

		return STATE_UNKNOWN;
	        }


	/* initialize alarm signal handling */
	signal(SIGALRM,alarm_handler);

	/* set socket timeout */
	alarm(timeout);

	time(&start_time);

	/* try to connect to the host at the given port number */
	result=my_connect(server_name,server_port,"tcp",&sd);

	/* we connected, so close connection before exiting */
	if(result==STATE_OK){

		/* watch for the FTP connection string */
		result=recv(sd,&raw_server_time,sizeof(raw_server_time),0);

		/* return a WARNING status if we couldn't read any data */
		if(result==-1){
			printf("recv() failed\n");
			result=STATE_WARNING;
		        }

		else{
			time(&local_time);

			time(&end_time);

			result=STATE_OK;

			if(check_critical_time==TRUE && (end_time-start_time)>critical_time)
				result=STATE_CRITICAL;
			else if(check_warning_time==TRUE && (end_time-start_time)>warning_time)
				result=STATE_WARNING;

			if(result!=STATE_OK)
				printf("Time server problem - %d second response time\n",(int)(end_time-start_time));

			else{

			        server_time=(unsigned long)(ntohl(raw_server_time)-((unsigned long)UNIX_EPOCH));

				if(server_time>local_time)
					diff_time=server_time-local_time;
				else
					diff_time=local_time-server_time;

				if(check_critical_diff==TRUE && diff_time>critical_diff)
					result=STATE_CRITICAL;
				else if(check_warning_diff==TRUE && diff_time>warning_diff)
					result=STATE_WARNING;

				printf("Time server %s - %lu sec. time difference\n",(result==STATE_OK)?"ok":"problem",diff_time);
		                }

		        }

		/* close the connection */
		close(sd);
	        }

	/* reset the alarm */
	alarm(0);

	return result;
        }



int process_arguments(int argc, char **argv){
	int x;


	/* no options were supplied */
	if(argc<2)
		return ERROR;

	/* first option is always the server name/address */
	strcpy(server_name,argv[1]);

	for(x=3;x<=argc;x++){

		if(!strcmp(argv[x-1],"-p")){
			if(x<argc){
				server_port=atoi(argv[x]);
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-to")){
			if(x<argc){
				timeout=atoi(argv[x]);
				if(timeout<=0)
					return ERROR;
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-wt")){
			if(x<argc){
				warning_time=atoi(argv[x]);
				check_warning_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-ct")){
			if(x<argc){
				critical_time=atoi(argv[x]);
				check_critical_time=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-wd")){
			if(x<argc){
				warning_diff=strtoul(argv[x],NULL,10);
				check_warning_diff=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }
		else if(!strcmp(argv[x-1],"-cd")){
			if(x<argc){
				critical_diff=strtoul(argv[x],NULL,10);
				check_critical_diff=TRUE;
				x++;
			        }
			else
				return ERROR;
		        }
		else
			return ERROR;
	        }

	return OK;
        }



int my_connect(char *host_name,int port,char *protocol,int *sd){
	struct sockaddr_in servaddr;
	struct hostent *hp;
	struct protoent *ptrp;
	int result;

	bzero((char *)&servaddr,sizeof(servaddr));
	servaddr.sin_family=AF_INET;
	servaddr.sin_port=htons(port);

	/* try to bypass using a DNS lookup if this is just an IP address */
	if(!inet_aton(host_name,&servaddr.sin_addr)){

		/* else do a DNS lookup */
		hp=gethostbyname((const char *)host_name);
		if(hp==NULL){
			printf("Invalid host name '%s'\n",host_name);
			return STATE_UNKNOWN;
		        }

		memcpy(&servaddr.sin_addr,hp->h_addr,hp->h_length);
	        }

	/* map TCP transport protocol name to protocol number */
	if(((int)(ptrp=getprotobyname(protocol)))==(int)0){
		printf("Cannot map \"%s\" to protocol number\n",protocol);
		return STATE_UNKNOWN;
	        }

	/* create a socket */
	*sd=socket(PF_INET,SOCK_STREAM,ptrp->p_proto);
	if(*sd<0){
		printf("Socket creation failed\n");
		return STATE_UNKNOWN;
	        }

	/* open a connection */
	time(&start_time);
	result=connect(*sd,(struct sockaddr *)&servaddr,sizeof(servaddr));
	if(result<0){
		switch(errno){  
		case ECONNREFUSED:
			printf("Connection refused by host\n");
			break;
		case ETIMEDOUT:
			printf("Timeout while attempting connection\n");
			break;
		case ENETUNREACH:
			printf("Network is unreachable\n");
			break;
		default:
			printf("Connection refused or timed out\n");
		        }

		return STATE_CRITICAL;
	        }

	return STATE_OK;
        }



void alarm_handler(int sig){

	printf("Socket timeout after %d seconds\n",timeout);

	exit(STATE_CRITICAL);
        }
